package com.qmth.sop.server.api;

import com.google.common.collect.ImmutableMap;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.sop.business.bean.report.QualityAnalyseReportBean;
import com.qmth.sop.business.service.QualityAnalyseReportService;
import com.qmth.sop.common.contant.SystemConstant;
import com.qmth.sop.common.util.Result;
import com.qmth.sop.common.util.ResultUtil;
import io.swagger.annotations.*;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;
import java.util.Map;
import java.util.Objects;

@Api(tags = "质量监控分析Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + SystemConstant.PREFIX_URL_QUALITY_ANALYSE)
public class QualityAnalyseReportController {

    public static final String IN_FLUENCE_DEGREE_A_TOTAL = "influenceDegreeATotal";

    public static final String IN_FLUENCE_DEGREE_B_TOTAL = "influenceDegreeBTotal";

    public static final String IN_FLUENCE_DEGREE_C_TOTAL = "influenceDegreeCTotal";

    public static final String IN_FLUENCE_DEGREE_D_TOTAL = "influenceDegreeDTotal";

    public static final String IN_FLUENCE_DEGREE_A_RATE_TOTAL = "influenceDegreeARateTotal";

    public static final String IN_FLUENCE_DEGREE_B_RATE_TOTAL = "influenceDegreeBRateTotal";

    public static final String IN_FLUENCE_DEGREE_C_RATE_TOTAL = "influenceDegreeCRateTotal";

    public static final String IN_FLUENCE_DEGREE_D_RATE_TOTAL = "influenceDegreeDRateTotal";

    @Resource
    QualityAnalyseReportService qualityAnalyseReportService;

    @ApiOperation(value = "大区质量问题统计")
    @RequestMapping(value = "/area_manager/report", method = RequestMethod.POST)
    @ApiResponses({ @ApiResponse(code = 200, message = "质量监控分析报表", response = QualityAnalyseReportBean.class) })
    public Result areaManagerReport(@ApiParam(value = "服务单元id", required = true) @RequestParam Long serviceId) {
        return ResultUtil.ok(this.calculateRate(qualityAnalyseReportService.areaQualityReport(serviceId)));
    }

    @ApiOperation(value = "人力商质量问题统计")
    @RequestMapping(value = "/human_supplier/report", method = RequestMethod.POST)
    @ApiResponses({ @ApiResponse(code = 200, message = "质量监控分析报表", response = QualityAnalyseReportBean.class) })
    public Result humanSupplierReport(@ApiParam(value = "服务单元id", required = true) @RequestParam Long serviceId) {
        return ResultUtil.ok(this.calculateRate(qualityAnalyseReportService.humanSupplierQualityReport(serviceId)));
    }

    @ApiOperation(value = "质量问题原因分类")
    @RequestMapping(value = "/type/report", method = RequestMethod.POST)
    @ApiResponses({ @ApiResponse(code = 200, message = "质量监控分析报表", response = QualityAnalyseReportBean.class) })
    public Result typeReport(@ApiParam(value = "服务单元id", required = true) @RequestParam Long serviceId) {
        return ResultUtil.ok();
    }

    /**
     * 计算比率
     *
     * @param qualityAnalyseReportBeanList
     * @return
     */
    protected Map<String, Object> calculateRate(List<QualityAnalyseReportBean> qualityAnalyseReportBeanList) {
        Integer total = qualityAnalyseReportBeanList.stream().mapToInt(s -> s.getSum()).sum();
        Integer influenceDegreeATotal = qualityAnalyseReportBeanList.stream().mapToInt(s -> s.getInfluenceDegreeA())
                .sum();
        Integer influenceDegreeBTotal = qualityAnalyseReportBeanList.stream().mapToInt(s -> s.getInfluenceDegreeB())
                .sum();
        Integer influenceDegreeCTotal = qualityAnalyseReportBeanList.stream().mapToInt(s -> s.getInfluenceDegreeC())
                .sum();
        Integer influenceDegreeDTotal = qualityAnalyseReportBeanList.stream().mapToInt(s -> s.getInfluenceDegreeD())
                .sum();
        BigDecimal influenceDegreeARateTotal = new BigDecimal(influenceDegreeATotal);
        BigDecimal influenceDegreeBRateTotal = new BigDecimal(influenceDegreeBTotal);
        BigDecimal influenceDegreeCRateTotal = new BigDecimal(influenceDegreeCTotal);
        BigDecimal influenceDegreeDRateTotal = new BigDecimal(influenceDegreeDTotal);

        if (Objects.nonNull(total) && total.intValue() > 0) {
            influenceDegreeARateTotal = influenceDegreeARateTotal.divide(new BigDecimal(total), 2,
                    BigDecimal.ROUND_HALF_UP).multiply(SystemConstant.RATE).setScale(2, BigDecimal.ROUND_HALF_UP);
            influenceDegreeBRateTotal = influenceDegreeBRateTotal.divide(new BigDecimal(total), 2,
                    BigDecimal.ROUND_HALF_UP).multiply(SystemConstant.RATE).setScale(2, BigDecimal.ROUND_HALF_UP);
            influenceDegreeCRateTotal = influenceDegreeCRateTotal.divide(new BigDecimal(total), 2,
                    BigDecimal.ROUND_HALF_UP).multiply(SystemConstant.RATE).setScale(2, BigDecimal.ROUND_HALF_UP);
            influenceDegreeDRateTotal = influenceDegreeDRateTotal.divide(new BigDecimal(total), 2,
                    BigDecimal.ROUND_HALF_UP).multiply(SystemConstant.RATE).setScale(2, BigDecimal.ROUND_HALF_UP);
        }
        return new ImmutableMap.Builder<String, Object>().put(ResourceSecurityReportController.TOTAL, total)
                .put(IN_FLUENCE_DEGREE_A_TOTAL, influenceDegreeATotal)
                .put(IN_FLUENCE_DEGREE_B_TOTAL, influenceDegreeBTotal)
                .put(IN_FLUENCE_DEGREE_C_TOTAL, influenceDegreeCTotal)
                .put(IN_FLUENCE_DEGREE_D_TOTAL, influenceDegreeDTotal)
                .put(IN_FLUENCE_DEGREE_A_RATE_TOTAL, influenceDegreeARateTotal)
                .put(IN_FLUENCE_DEGREE_B_RATE_TOTAL, influenceDegreeBRateTotal)
                .put(IN_FLUENCE_DEGREE_C_RATE_TOTAL, influenceDegreeCRateTotal)
                .put(IN_FLUENCE_DEGREE_D_RATE_TOTAL, influenceDegreeDRateTotal)
                .put(ResourceSecurityReportController.RECORDS, qualityAnalyseReportBeanList).build();
    }
}
